//==============================================================================
// a dynamic Light class that supports strobing and pulsing at a certain rate.
// Can be triggered ON/OFF.
// Works with states.
// state Pulsing:  Light will gradually increase/decrease a number of seconds as
//                 stated in PulseTime and stay ON/OFF as stated in Frequency
//
// state Strobing: Light will gradually increase/decrease a number of seconds as
//                 stated in PulseTime
////==============================================================================

class MSBLight extends Light;


var() float FrequencyOn;  // On Time of this Light in Hz
var() float FrequencyOff; // OFF Time of this Light in Hz
var() float PulseTime;  // RampUp/RampDown Time of this Light in Seconds
var() bool bActive;  // ON/OFF switch
var() name TriggerOn;
var() name TriggerOff;


//------------------------------------------------------------------------------
// internal calculation options

var float InitialBrightness; // stores Initial Brighthness when we Ramp
var float Alpha; // actual Amount of Light that' being added/subtracted each tick
var float Direction; // determines RampUp/Down
var name DesiredState;

//------------------------------------------------------------------------------
// networking stuff  and color changing options

var float LightRad;
var float LightB;
var float LightH;
var float LightS;

var config array<string> Aniname;
var array<string> ButtonList;
var array<string> Properties;


replication
{

reliable if (ROLE==ROLE_Authority)
LightRad, InitialBrightness;
}


//------------------------------------------------------------------------------
// initialize, store initialBrightness, set Active state: waiting for bActive=OFF

simulated function BeginPlay()
{
	InitialBrightness = LightBrightness; //how bright are we at the beginning?

    Direction = 1;

    DesiredState=InitialState;

    if (!bActive) InitialState='waiting';

}




//------------------------------------------------------------------------------
// Triggers Light ON and OFF

simulated function Trigger(actor Other, pawn EventInstigator)
{
 bActive=!bActive;

 if (bActive)
 {
  if (DesiredState=='PulseLight')
  {
    //Alpha=0;
    Direction=1;
    GotoState('PulseLight','RampUp');
  }
 }

 if (bActive && DesiredState=='StrobeLight')
 {
  GotoState('StrobeLight');
 }

 if (!bActive)
 {
  GotoState('waiting');
 }


}

//------------------------------------------------------------------------------
// not really sure about this one...
// Can we leave this out? Let's try...
/*
//function UnTrigger(actor Other, pawn EventInstigator)
{
 GotoState('waiting');
}
*/

//------------------------------------------------------------------------------
// idle state

state() waiting
{

//------------------------------------------------------------------------------
// Turn the Light Off everytime we switch to waiting

 simulated function TurnOFFEnd()
  {
   LightBrightness=1;
  // Log("Now Lights OFF");
  }

 Begin:

      TurnOFFEnd();
      Disable('Tick'); //Disable changing
}


//------------------------------------------------------------------------------
// pulsing - we begin with RampDown

state() PulseLight
{

//------------------------------------------------------------------------------
// Tick from PulseLight state  - gradually decrease the LightBrightness

    simulated function Tick( float DeltaTime )
    {
	Alpha += Direction * DeltaTime / PulseTime;
	if( Alpha > 1.0 )
	  {
      Alpha = 1.0;
      Disable( 'Tick' );
	  }
	else if( Alpha < 0.0 )
	  {
	  Alpha = 0.0;
	  Disable( 'Tick' );
	  }
    LightBrightness = Alpha * InitialBrightness;

	if (ROLE==ROLE_Authority)                    //Added for Network functionality
      {
      LightRad=LightRadius;
       }
    else
      {
      LightRadius=LightRad;
      }
    }




 RampUp:

     Direction=1;
     SetTimer (PulseTime, false);
     Enable('Tick');
     sleep(PulseTime);
     Disable('Tick');
     sleep(1/FrequencyOn);
     self.triggerEvent(TriggerOn,self,None);


 RampDown:

     Direction=-1;
     Enable('Tick');
     sleep(PulseTime);
     Disable('Tick');
     sleep(1/FrequencyOff);
     self.triggerEvent(TriggerOff,self,None);

 Begin:

     goto ('RampUp');
}


//------------------------------------------------------------------------------
// strobing state

state() StrobeLight
{

//------------------------------------------------------------------------------
// from StrobeLight state - set the LightBrightness to stored InitialBrightness

 simulated function TurnON()
  {
   LightBrightness=InitialBrightness;
   self.triggerEvent(TriggerOn,self,None);
  // Log("Now Lights ON");
  }

//------------------------------------------------------------------------------
// from StrobeLight state - set LightBrightness to 1

  simulated function TurnOFF()
  {
   LightBrightness=1;
   self.triggerEvent(TriggerOff,self,None);
   // Log("Now Lights OFF");
  }

  simulated function Tick (float DeltaTime)
  {
	if (ROLE==ROLE_Authority)                    //Added for Network functionality
      {
      LightRad=LightRadius;
      }
    else
      {
      LightRadius=LightRad;
      }
  }

 Begin:

     TurnON();
     sleep(1/FrequencyOn);
     TurnOFF();
     sleep(1/FrequencyOFF);
     goto ('Begin');

}

//------------------------------------------------------------------------------
// default

defaultproperties
{

ButtonList(0)="MSBPropertyButtons.MSBPropertyButtonMove"
Properties(0)="Dummy"
ButtonList(1)="MSBPropertyButtons.MSBPropertyButtonDuplicate"
Properties(1)="Dummy"
ButtonList(2)="MSBPropertyButtons.MSBPropertyButtonAdvancedProps"
Properties(2)="Dummy"
ButtonList(3)="MSBPropertyButtons.MSBPropertyButtonText"
Properties(3)="LightBrightness"
ButtonList(4)="MSBPropertyButtons.MSBPropertyButtonText"
Properties(4)="LightSaturation"
ButtonList(5)="MSBPropertyButtons.MSBPropertyButtonText"
Properties(5)="LightRadius"

bActive=true
bStatic=False
bHidden=False
bDynamicLight=True
bNodelete=False

bAlwaysRelevant=true
bReplicateMovement=true
bUpdateSimulatedPosition=true
RemoteRole=ROLE_SimulatedProxy

bMovable=True
PulseTime=0.2
FrequencyOn=10
FrequencyOff=10

DrawType=DT_Sprite
CollisionRadius=16.0
CollisionHeight=16.0

bUseCylinderCollision=true
bCollideActors=True
bBlockActors=False
bCollideWorld=True
bProjTarget=True
}
