//-----------------------------------------------------------
//
//-----------------------------------------------------------
class MSBVoxelPawn extends MSBPawn;

var vector MasterSize;
var array<MSBMasterBlock> Blocks;
var array<MSBMasterBlock> FaceBlocks;                      //can mimic and talk
var array<name> FaceBone;                      //names of face bones

var bool bLoading;                             //to determine who called the timer

var vector mOpen[8];
var vector mClosed[8];

event PostBeginPlay()
{
super.PostBeginPlay();
setMSBMasterBlocks();
initMimics();
}

// *******************************************************************************
// Lipsyncing
// *******************************************************************************

function initMimics()
{
local int i;

getFace();

mOpen[0].Z=0.2;
mOpen[1].Z=0.2;
mOpen[2].Z=0.2;

mOpen[3].Z=0;

mOpen[4].Z=-0.2;
mOpen[5].Z=-0.2;
mOpen[6].Z=-0.2;

mOpen[7].Z=0;

for (i=0;i<8;i++)
  if (FaceBlocks.Length>i && FaceBlocks[i]!=None)
    mClosed[i].Z=0;
}

function getFace()
{
local int i;

for (i=0;i<8;i++)
  {
  if (FaceBlocks[i]==None)
    FaceBlocks[i]=Spawn(class'MSBMasterBlock',self,,);
  FaceBlocks[i].bFaceBlock=true;
  FaceBlocks[i].Tag='Face';
  FaceBlocks[i].BoneNumber=i;
  FaceBlocks[i].SetDrawScale(0.05);
  FaceBlocks[i].SetDrawType(DT_None);
  AttachToBone(FaceBlocks[i],FaceBone[i]);
  }
}

simulated function AssignInitialPose()
{
local int i;
super.AssignInitialPose();
for (i=0;i<8;i++)
  if (FaceBlocks[i]!=None)
    AttachToBone(FaceBlocks[i],FaceBone[i]);
}


function Talk()
{
local int i;

for (i=0;i<8;i++)
  if (FaceBlocks[i]!=None)
    FaceBlocks[i].SetRelativeLocation(FRand()*mOpen[i]);

SetTimer(0.1,false);
}

function Mimic()
{
super.Mimic();
}

// *******************************************************************************
// Loading Voxels and cleaning up
// *******************************************************************************
function LoadCharacter()
{
LoadCharacterOne();
bLoading=true;
SetTimer(1,false);
}

function Timer()
{
local int i;
if (bLoading)
  {
  LoadCharacterTwo();
  bLoading=false;
  }
else

for (i=0;i<8;i++)
  if (FaceBlocks[i]!=None)
    FaceBlocks[i].SetRelativeLocation(mClosed[i]);
}

function LoadCharacterOne()
{
local MSBVoxelCharacterSave char;
local int bone;
local MSBMasterBlock Master;

char=Level.Game.LoadDataObject(class'MSBVoxelCharacterSave',content,"MSBVoxChar_"$content);
for(bone=0;bone<char.Parts.length;bone++)
  {
    Master=Blocks[bone];
    if (Master!=None)
      {
      //get rid of old blocks
      Master.clearBlocks();
      Master.E=Spawn(class'MSBEmitter',Master);
      Master.E.P.SpawnParticle(char.Parts[bone].Col.length);
      Master.SetCollision(false,false,false);
      Master.SetDrawType(DT_None);
      }
  }

for(bone=0;bone<char.Face.length;bone++)
  {
    Master=FaceBlocks[bone];
    if (Master!=None)
      {
      //get rid of old blocks
      Master.clearBlocks();
      Master.E=Spawn(class'MSBEmitter',Master);
      Master.E.P.SpawnParticle(char.Face[bone].Col.length);
      Master.SetCollision(false,false,false);
      Master.SetDrawType(DT_None);
      }
  }
}

function LoadCharacterTwo()
{
local MSBVoxelCharacterSave char;
local int bone,block;
local MSBMasterBlock Master;

char=Level.Game.LoadDataObject(class'MSBVoxelCharacterSave',content,"MSBVoxChar_"$content);
for(bone=0;bone<char.Parts.length;bone++)
  {
    Master=Blocks[bone];
    //set new blocks
    if (Master!=None)
     for (block=0;block<char.Parts[bone].Col.length;block++)
      {
      //size
      Master.setScale(char.Parts[Bone].drawScale[block],block);
      //color
      Master.setColor(char.Parts[bone].Col[block],block);
      //location
      Master.setParticleLocation(char.Parts[bone].BlockRelLocation[block], block);
      }
  }
for(bone=0;bone<char.Face.length;bone++)
  {
    Master=FaceBlocks[bone];
    Log(Master.Tag);
    //set new blocks
    if (Master!=None)
     for (block=0;block<char.Face[bone].Col.length;block++)
      {
      //size
      Master.setScale(char.Face[Bone].drawScale[block],block);
      //color
      Master.setColor(char.Face[bone].Col[block],block);
      //location
      Master.setParticleLocation(char.Face[bone].BlockRelLocation[block], block);
      }
  }
}

function setMSBMasterBlocks()
{
local coords C[26];
local int i;

C[0]=GetBoneCoords('Bip01');

C[1]=GetBoneCoords('Bip01 Pelvis');
C[2]=GetBoneCoords('bip01 Spine');
C[3]=GetBoneCoords('bip01 Spine1');
C[4]=GetBoneCoords('bip01 Spine2');
C[5]=GetBoneCoords('Bip01 Neck');

C[6]=GetBoneCoords('Bip01 R Clavicle');
C[7]=GetBoneCoords('Bip01 R UpperArm');
C[8]=GetBoneCoords('Bip01 R Forearm');
C[9]=GetBoneCoords('Bip01 R Hand');
C[10]=GetBoneCoords('Bip01 R Finger');
C[11]=GetBoneCoords('Bone_weapon');

C[12]=GetBoneCoords('Bip01 L Clavicle');
C[13]=GetBoneCoords('Bip01 L UpperArm');
C[14]=GetBoneCoords('Bip01 L Forearm');
C[15]=GetBoneCoords('Bip01 L Hand');
C[16]=GetBoneCoords('Bip01 L Finger');

C[17]=GetBoneCoords('bip01 R Thigh');
C[18]=GetBoneCoords('Bip01 R Calf');
C[19]=GetBoneCoords('Bip01 R Foot');
C[20]=GetBoneCoords('Bip01 R Toe0');

C[21]=GetBoneCoords('bip01 L Thigh');
C[22]=GetBoneCoords('Bip01 L Calf');
C[23]=GetBoneCoords('Bip01 L Foot');
C[24]=GetBoneCoords('Bip01 L Toe0');

C[25]=GetBoneCoords('Bip01 Head');

for (i=0;i<26;i++)
  {
  Blocks[i]=Spawn(class'MSBMasterBlock',self,,C[i].Origin);
  Blocks[i].BoneNumber=i;
  Blocks[i].P=self;
  }
}

function clearPawn()
{
local int i;

for (i=0;i<26;i++)
  {
  Blocks[i].clearBlocks();
  Blocks[i].Destroy();
  if (FaceBlocks.length>i)
   if (FaceBlocks[i]!=None)
     FaceBlocks[i].Destroy();
  }
}

function scaleMe(float chgScale)
{
local int i;

for (i=0;i<Blocks.length;i++)
  {
  Blocks[i].scaleMe(chgScale);
  if (FaceBlocks.length>i && FaceBlocks[i]!=None)
    FaceBlocks[i].scaleMe(chgScale);
  }
}

DefaultProperties
{
Mesh=Mesh'MSB_Reference.complex'                                 //base animation resource
UTMesh=Mesh'MSB_Reference.complex'                                 //base animation resource

///************************************************************
//Buttonstuff is already implemented in Base class MSBPawn!!! *
///************************************************************
bSkinSwap=false
bTalkable=false

bCollideActors=true
CollisionHeight=48.0
Skins(0)=Material'MovieSandboxTextures.Transparent'
Skins(1)=Material'MovieSandboxTextures.Transparent'
Skins(2)=Material'MovieSandboxTextures.Transparent'
Skins(3)=Material'MovieSandboxTextures.Transparent'

FaceBone(0)=BN-UpperLipRT
FaceBone(1)=BN-UpperLipMI
FaceBone(2)=BN-UpperLipLT


FaceBone(3)=BN-LipCornerRT
FaceBone(4)=BN-LowerLipRT
FaceBone(5)=BN-LowerLipMI
FaceBone(6)=BN-LowerLipLT
FaceBone(7)=BN-LipCornerLT

FaceBone(8)=BN-Jaw
FaceBone(9)=BN-TongueBack
FaceBone(10)=BN-TongueFront

FaceBone(11)=BN-CheekRT
FaceBone(12)=BN-CheekLT

FaceBone(13)=BN-LowerEyeRT
FaceBone(14)=BN-EyeRT
FaceBone(15)=BN-UpperEyeRT
FaceBone(16)=BN-BrowRT

FaceBone(17)=BN-LowerEyeLT
FaceBone(18)=BN-EyeLT
FaceBone(19)=BN-UpperEyeLT
FaceBone(20)=BN-BrowLT
}
